diff --git a/fluid/Fl_Type.cxx b/fluid/Fl_Type.cxx
index f8cdfa1..f44c39c 100644
--- a/fluid/Fl_Type.cxx
+++ b/fluid/Fl_Type.cxx
@@ -258,6 +258,7 @@ void deselect() {
 
 Fl_Type *Fl_Type::first;
 Fl_Type *Fl_Type::last;
+Fl_Type *Fl_Type::paste_after_ = 0;
 
 static void Widget_Browser_callback(Fl_Widget *o,void *) {
   ((Widget_Browser *)o)->callback();
@@ -581,15 +582,20 @@ void Fl_Type::remove_child(Fl_Type*) {}
 
 // add a list of widgets as a new child of p:
 void Fl_Type::add(Fl_Type *p) {
-  if (p && parent == p) return;
-  undo_checkpoint();
-  parent = p;
+  if (p && parent == p) return;         // don't add to self
+  undo_checkpoint();                    // save undo state before modify
+  parent = p;                           // p will be our new parent
+  // Find end of our own linked list
   Fl_Type *end = this;
   while (end->next) end = end->next;
-  Fl_Type *q;
+  // Find last sibling in parent so we can append to that.
+  // If caller specified a paste_after() sibling, find that instead.
+  Fl_Type *q = 0;                       // where in list to add() ourself
   int newlevel;
   if (p) {
-    for (q = p->next; q && q->level > p->level; q = q->next) {/*empty*/}
+    for (q = p->next; q && q->level > p->level; q = q->next) {
+      if ( q == paste_after_ ) break;	// found specific child? done with search
+    }
     newlevel = p->level+1;
   } else {
     q = 0;
@@ -598,16 +604,19 @@ void Fl_Type::add(Fl_Type *p) {
   for (Fl_Type *t = this->next; t; t = t->next) t->level += (newlevel-level);
   level = newlevel;
   if (q) {
+    // Insert self into linked list at q's position
     prev = q->prev;
     prev->next = this;
     q->prev = end;
     end->next = q;
   } else if (first) {
+    // Children exist? append after last child
     prev = last;
     prev->next = this;
     end->next = 0;
     last = end;
   } else {
+    // No children? We become first
     first = this;
     last = end;
     prev = end->next = 0;
diff --git a/fluid/Fl_Type.h b/fluid/Fl_Type.h
index 6794bef..ff404b8 100644
--- a/fluid/Fl_Type.h
+++ b/fluid/Fl_Type.h
@@ -46,6 +46,7 @@ class Fl_Type {
   friend Fl_Widget *make_type_browser(int,int,int,int,const char *);
   friend class Fl_Window_Type;
   virtual void setlabel(const char *); // virtual part of label(char*)
+  static Fl_Type *paste_after_;	// paste() appends after this widget
 
 protected:
 
@@ -160,6 +161,7 @@ public:
 
   const char* class_name(const int need_nest) const;
   const class Fl_Class_Type* is_in_class() const;
+  static void paste_after(Fl_Type *p) { paste_after_ = p; }
 };
 
 class Fl_Function_Type : public Fl_Type {
diff --git a/fluid/fluid.cxx b/fluid/fluid.cxx
index c80d351..4217f34 100644
--- a/fluid/fluid.cxx
+++ b/fluid/fluid.cxx
@@ -831,6 +831,21 @@ void delete_cb(Fl_Widget *, void *) {
 
 extern int force_parent;
 
+// Return the last selected item, or NULL if none
+Fl_Type *last_selected() {
+  if ( !Fl_Type::current ) return 0;
+  Fl_Type *qlast = Fl_Type::current->parent;
+  if ( qlast && !qlast->selected ) {
+    // Look for selected children
+    for ( Fl_Type *q = qlast->first; q && q->next; q = q->next ) {
+      if ( q->selected ) {
+        qlast = q->next ? q->next : q;
+      }
+    }
+  }
+  return qlast;
+}
+
 void paste_cb(Fl_Widget*, void*) {
   //if (ipasteoffset) force_parent = 1;
   pasteoffset = ipasteoffset;
@@ -838,9 +853,13 @@ void paste_cb(Fl_Widget*, void*) {
   if (gridy>1) pasteoffset = ((pasteoffset-1)/gridy+1)*gridy;
   undo_checkpoint();
   undo_suspend();
+  // Find last selected item, configure to paste after it
+  Fl_Type *after = last_selected();
+  Fl_Type::paste_after(after);
   if (!read_file(cutfname(), 1)) {
     fl_message("Can't read %s: %s", cutfname(), strerror(errno));
   }
+  Fl_Type::paste_after(0);
   undo_resume();
   pasteoffset = 0;
   ipasteoffset += 10;
